package com.ebaiyihui.family.doctor.server.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.RedisStringCommands;
import org.springframework.data.redis.core.*;
import org.springframework.data.redis.core.types.Expiration;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Component;
import redis.clients.jedis.JedisCluster;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName: RedisUtil
 * @Author:yanliang
 * @Date:2024/4/26 09:49
 * @Description
 */
@Slf4j
@Component("redisUtil")
public final class RedisUtil {

    @Resource
    private RedisTemplate<String, String> redisTemplate;


    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @Autowired
    public JedisCluster jedisCluster;

    public void setJedisCluster(JedisCluster jedisCluster) {
        this.jedisCluster = jedisCluster;
    }

    /**
     * 根据key获取value
     *
     * @param key
     * @return
     */
    public String get(final String key) {
        String result = redisTemplate.execute((RedisCallback<String>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            byte[] value = connection.get(serializer.serialize(key));
            return serializer.deserialize(value);
        });
        return result;
    }

    /**
     * 存储
     *
     * @param key
     * @param value
     * @return
     */
    public boolean set(final String key, final String value) {
        boolean result = redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.set(serializer.serialize(key), serializer.serialize(value));
            return true;
        });
        return result;
    }

    /**
     * 过期存储
     *
     * @param key
     * @param value
     * @param expirationTime 过期时间
     * @return
     */
    public boolean set(final String key, final String value, final long expirationTime) {
        boolean result = redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.set(serializer.serialize(key), serializer.serialize(value), Expiration.seconds(expirationTime),
                    RedisStringCommands.SetOption.UPSERT);
            return true;
        });
        return result;
    }

    /**
     * 删除缓存
     *
     * @param key
     * @return
     */
    public boolean delete(final String key) {
        boolean result = redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.del(serializer.serialize(key));
            return true;
        });
        return result;
    }

    /**
     * 哈希 添加
     *
     * @param key
     * @param hashKey
     * @param value
     */
    public boolean setHash(String key, String hashKey, String value) {
        return redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.hSet(serializer.serialize(key), serializer.serialize(hashKey), serializer.serialize(value));
            return true;
        });
    }

    /**
     * 哈希获取数据
     *
     * @param key
     * @param hashKey
     * @return
     */
    public Object getHash(String key, String hashKey) {
        return redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.hGet(serializer.serialize(key), serializer.serialize(hashKey));
            return true;
        });
    }

    /**
     * 列表添加
     *
     * @param key
     * @param value
     */
    public boolean rPush(String key, String value) {
        return redisTemplate.execute((RedisCallback<Boolean>) connection -> {
            RedisSerializer<String> serializer = redisTemplate.getStringSerializer();
            connection.rPush(serializer.serialize(key), new byte[][]{serializer.serialize(value)});
            return true;
        });
    }

    /**
     * 列表获取
     *
     * @param key
     * @param l
     * @param l1
     * @return
     */
    public List<?> lRange(String key, long l, long l1) {
        ListOperations<String, ?> list = redisTemplate.opsForList();
        return list.range(key, l, l1);
    }

    /**
     * 集合添加
     *
     * @param key
     * @param value
     */
    public void add(String key, String value) {
        redisTemplate.opsForSet().add(key, value);
    }

    /**
     * 集合获取
     *
     * @param key
     * @return
     */
    public Set<String> setMembers(String key) {
        return redisTemplate.opsForSet().members(key);
    }

    /**
     * 有序集合添加
     *
     * @param key
     * @param value
     * @param scoure
     */
    public void zAdd(String key, String value, double scoure) {
        ZSetOperations<String, String> zset = redisTemplate.opsForZSet();
        zset.add(key, value, scoure);
    }

    /**
     * 有序集合获取
     *
     * @param key
     * @param scoure
     * @param scoure1
     * @return
     */
    public Set<String> rangeByScore(String key, double scoure, double scoure1) {
        ZSetOperations<String, String> zset = redisTemplate.opsForZSet();
        return zset.rangeByScore(key, scoure, scoure1);
    }

    /**
     * 获取Redis锁
     *
     * @param key      锁键值
     * @param lockTime 锁时间
     * @return 是否获取成功
     */
    public Boolean getLock(String key, Long lockTime) {
        String lock = this.get(key);
        if (lock == null) {
            this.set(key, "lock", lockTime);
            return true;
        }
        return false;
    }

    /**
     * 删除redis锁
     *
     * @param key
     * @return
     */
    public boolean del(String key) {
        return stringRedisTemplate.delete(key);
    }


    private static final Long RELEASE_SUCCESS = 1L;
    private static final String LOCK_SUCCESS = "OK";

    private static final long WAIT_TIMEOUT = 10; // 获取锁的最大等待时间，单位为秒

    /**
     * 获取分布式锁
     *
     * @param lockKey    锁
     * @param requestId  请求标识
     * @param expireTime 超期时间
     * @return 是否获取成功
     */
    public boolean lock(String lockKey, String requestId, int expireTime) {
        try {
            expireTime = 1000 * expireTime;
            String result = jedisCluster.set(lockKey, requestId, "NX", "PX", expireTime);
            return LOCK_SUCCESS.equals(result);
        } catch (Exception e) {
            log.error("tryLock is error", e);
            return false;
        }
    }

    /**
     * 尝试获取分布式锁
     *
     * @param lockKey    锁
     * @param requestId  请求标识
     * @param expireTime 超期时间
     * @return 是否获取成功
     */
    public boolean tryLock(String lockKey, String requestId, int expireTime) {
        long start = System.currentTimeMillis();
        try {
            while (true) {
                expireTime = 1000 * expireTime;
                String result = jedisCluster.set(lockKey, requestId, "NX", "PX", expireTime);
                if (LOCK_SUCCESS.equals(result)) {
                    return true; // 获取锁成功
                }
                // 判断是否超时
                if (System.currentTimeMillis() - start > WAIT_TIMEOUT * 1000) {
                    return false;
                }
                // 休眠一段时间后重试
                TimeUnit.MILLISECONDS.sleep(100);
            }

        } catch (Exception e) {
            log.error("tryLock is error", e);
            return false;
        }
    }

    /**
     * 释放分布式锁
     *
     * @param lockKey   锁
     * @param requestId 请求标识
     * @return 是否释放成功
     */
    public boolean unLock(String lockKey, String requestId) {
        try {
            //eval命令执行Lua代码的时候，Lua代码将被当成一个命令去执行，并且直到eval命令执行完成，Redis才会执行其他命令
            String script = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
            Object result = jedisCluster.eval(script, Collections.singletonList(lockKey), Collections.singletonList(requestId));
            return RELEASE_SUCCESS.equals(result);
        } catch (Exception e) {
            log.error("unLock is error", e);
            return false;
        }
    }
}

